luanet.load_assembly("System");

local types = {};
types["Regex"] = luanet.import_type("System.Text.RegularExpressions.Regex");

function NavigateToReceiptPage()
	local browser = GetBrowser();

	local aCollection = browser.Document:GetElementsByTagName("a");
	LogDebug("Count of Anchor tags is: " .. aCollection.Count);
	local receiptButton = nil;

	local aCollectionEnumerator = aCollection:GetEnumerator();
	while aCollectionEnumerator:MoveNext() do
		local element = aCollectionEnumerator.Current;
		if element:GetAttribute("className") ~= "" then
			LogDebug("Found anchor class: " ..element:GetAttribute("className"));
			if string.match(element:GetAttribute("className"), "usa%-button%-primary") then
				receiptButton = element;
				break;
			end
		end
	end

	local link = receiptButton:GetAttribute("href");
	Navigate(link);
end

function OnPageByTitle(titleString)
	LogDebug("Checking if on order page by title: "..titleString);

	local onOrderPage = false;

	local browser = GetBrowser();
	local titleCollection = browser.Document:GetElementsByTagName("Title");
	LogDebug("Count of Title tags is: " .. titleCollection.Count);

	local titleCollectionEnumerator = titleCollection:GetEnumerator();
	while titleCollectionEnumerator:MoveNext() do
		local element = titleCollectionEnumerator.Current;
		LogDebug("Found Title: " .. element.InnerText);
		if string.match(element.InnerText,titleString) then
			LogDebug("Title matches");
			onOrderPage = true;
			break;
		end
	end

	if onOrderPage then
		LogDebug("On ("..titleString..") page: true");
	else
		LogDebug("On ("..titleString..") page: false");
	end

	return onOrderPage;
end

function OnPMIDSearchPage()
	return OnPageByTitle("Docline PMID Order %- Find a PMID");
end

function OnPMIDOrderPage()
	return OnPageByTitle("Docline PMID Order %- Place Order");
end

function OnManualOrderPage()
	return OnPageByTitle("Docline Manual Order %- Place Order");
end

function NavigateToPlaceRequest()
	LogDebug("Navigating to Find a PMID form.");
	ClickNavLink("Place request - PMID");
end

function SearchPMID()
	local browser = GetBrowser();

	local pubmedIdElement = browser.Document:GetElementById("id_pmids");

	local pmid = FieldTranslator.GetTranslationResults(FieldTranslator, "PubMedSearchID").Value;

	pubmedIdElement.SetAttribute(pubmedIdElement, "value", pmid);
	if pmid ~= "" then
		browser.Document:GetElementById("submit-id-submit"):InvokeMember("click");
	end
end

function PopulateSearchForm()
	LogDebug("Populating Search Form");
	local browser = GetBrowser();
	local currentValue = nil;

	--Authorized By
	-- This is autofilled in by the Docline login information. We are overriding it.
	browser.Document:GetElementById("id_authorized_by").SetAttribute(browser.Document:GetElementById("id_authorized_by"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "AuthorizedBy").Value);

	--Max Cost
	-- This is autofilled in by the Docline login information. Only override if one is present
	local maxCost = FieldTranslator.GetTranslationResults(FieldTranslator, "MaxCost").Value;
	if maxCost ~= nil then
		browser.Document:GetElementById("id_max_cost").SetAttribute(browser.Document:GetElementById("id_max_cost"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "MaxCost").Value);
	end

	--Patron Name
	browser.Document:GetElementById("id_patron_name").SetAttribute(browser.Document:GetElementById("id_patron_name"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PatronName").Value);

	--Need By Date
	local needBy = FieldTranslator.GetTranslationResults(FieldTranslator, "NeedBy").Value;
	if needBy ~= nil and needBy ~= "" then
		browser.Document:GetElementById("id_need_by_date").SetAttribute(browser.Document:GetElementById("id_need_by_date"), "value", needBy);
	end

	--Copyright Compliance
	local copyrightComp = FieldTranslator.GetTranslationResults(FieldTranslator, "CopyrightCompliance").Value;
	if copyrightComp ~= nil and copyrightComp ~= "" then
		browser.Document:GetElementById("id_copyright_compliance").SetAttribute(browser.Document:GetElementById("id_copyright_compliance"), "value", copyrightComp);
	end

	--Service Level
	local serviceLevel = FieldTranslator.GetTranslationResults(FieldTranslator, "ServiceLevel").Value;
	if serviceLevel ~= nil and serviceLevel ~= "" then
		browser.Document:GetElementById("id_service_level").SetAttribute(browser.Document:GetElementById("id_service_level"), "value", serviceLevel);
	end

	--Service Type
	local serviceType = FieldTranslator.GetTranslationResults(FieldTranslator, "ServiceType").Value;
	if serviceType ~= nil and serviceType ~= "" then
		browser.Document:GetElementById("id_request_type_requested").SetAttribute(browser.Document:GetElementById("id_request_type_requested"), "value", serviceType);
	end

	--Delivery Method
	local deliveryMethod = FieldTranslator.GetTranslationResults(FieldTranslator, "DeliveryMethod").Value;
	if deliveryMethod ~= nil and deliveryMethod ~= "" then
		browser.Document:GetElementById("id_delivery_method_requested").SetAttribute(browser.Document:GetElementById("id_delivery_method_requested"), "value", deliveryMethod);
	end

	--Odyssey
	browser.Document:GetElementById("id_delivery_odyssey").SetAttribute(browser.Document:GetElementById("id_delivery_odyssey"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "OdysseySystemID").Value);
end

function PopulateManualForm()
	PopulateSearchForm();

	LogDebug("Populating Manual Form");
	local browser = GetBrowser();

	--Journal/Book Title (required)
	browser.Document:GetElementById("id_journal_title").SetAttribute(browser.Document:GetElementById("id_journal_title"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Title").Value);

	--Book Author/Editor
	browser.Document:GetElementById("id_book_author").SetAttribute(browser.Document:GetElementById("id_book_author"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "JournalAuthor").Value);

	--Article or Chapter Title
	browser.Document:GetElementById("id_article_title").SetAttribute(browser.Document:GetElementById("id_article_title"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Article").Value);

	--Article or Chapter Author
	browser.Document:GetElementById("id_article_authors").SetAttribute(browser.Document:GetElementById("id_article_authors"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Author").Value);

	--Publisher
	browser.Document:GetElementById("id_journal_publisher").SetAttribute(browser.Document:GetElementById("id_journal_publisher"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Publisher").Value);

	--Place of Publication
	browser.Document:GetElementById("id_journal_place_of_publication").SetAttribute(browser.Document:GetElementById("id_journal_place_of_publication"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubPlace").Value);

	--ISSN/ISBN
	browser.Document:GetElementById("id_journal_issn").SetAttribute(browser.Document:GetElementById("id_journal_issn"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Issn").Value);

	--Year (required)
	browser.Document:GetElementById("id_article_year_string").SetAttribute(browser.Document:GetElementById("id_article_year_string"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PubYear").Value);

	--Volume
	browser.Document:GetElementById("id_article_volume_string").SetAttribute(browser.Document:GetElementById("id_article_volume_string"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Volume").Value);

	--Issue
	browser.Document:GetElementById("id_article_issue").SetAttribute(browser.Document:GetElementById("id_article_issue"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Issue").Value);

	--Pages
	browser.Document:GetElementById("id_article_pages").SetAttribute(browser.Document:GetElementById("id_article_pages"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "Pages").Value);

	--Patron Name
	browser.Document:GetElementById("id_patron_name").SetAttribute(browser.Document:GetElementById("id_patron_name"), "value", FieldTranslator.GetTranslationResults(FieldTranslator, "PatronName").Value);
end

function SubmitRequest()
	local browser = GetBrowser();
	local submitButton = browser.Document:GetElementById("submit-id-submit");
	if (submitButton ~= nil) then
		submitButton:InvokeMember("click");
	else
		LogDebug("Submit Button not found on page");
	end
end

function Finished()
	local browser = GetBrowser();

	--Grab the ID from the success page before navigating to the Receipts Page
	--Parse and grab the borrower and ID from the success page before navigating to the Receipts Page
	local anchorCollection = browser.Document:GetElementsByTagName("h3");
	LogDebug("Count of h3 tags is: ".. anchorCollection.Count);

	ParseSuccessPage(anchorCollection)

	_receiptTable["BorrowerTN"] = FieldTranslator.GetTranslationResults(FieldTranslator, "BorrowerTN").Value;

	NavigateToReceiptPage();
end

FieldTranslator = GetFieldTranslator()

RegisterPageHandler("title", "Docline PMID Order - Success", "Finished", true)
RegisterPageHandler("title", "Docline Manual Order - Success", "Finished", true)
RegisterPageHandler("title", "Docline Request", "ImportReceiptPage", true)