DoclineUrl = "https://newdocline.docline.gov/";

local Types = {};
Types["System.Object"] = luanet.import_type("System.Object");
Types["System.Dictionary"] = luanet.import_type("System.Dictionary");

_receiptTable = {};

function GetReceiptTable()
	return _receiptTable;
end

function trim(s)
   return (s:gsub("^%s*(.-)%s*$", "%1"))
end

function ClickNavLink(linkText)
	LogDebug("Attempting to click navigation link with text: " .. linkText);

	local updateLink;

	local browser = GetBrowser();
	local navCollection = browser.Document:GetElementsByTagName("nav");
	LogDebug("Count of NAV tags is: " .. navCollection.Count);
	local navMenu = nil;

	local navCollectionEnumerator = navCollection:GetEnumerator();
	while navCollectionEnumerator:MoveNext() do
		local element = navCollectionEnumerator.Current;
		LogDebug("Found NAV class: " .. element:GetAttribute("className"));
		if element:GetAttribute("className") == "usa-nav" then
			navMenu = element;
			break;
		end
	end

	if (navMenu ~= nil) then
		local navLinkCollection = navMenu:GetElementsByTagName("a");
		local navLinkCollectionEnumerator = navLinkCollection:GetEnumerator();
		while navLinkCollectionEnumerator:MoveNext() do
			local element = navLinkCollectionEnumerator.Current;
			if trim(element.InnerText) == linkText then
				LogDebug("Navigating to " .. linkText);
				element:InvokeMember("click");
			end
		end
	else
		LogDebug("nav menu was not found");
	end
end

function SelectInputOption(parentElement, inputType, selectedAttribute,  optionValue)
	LogDebug("Looking for " .. inputType .. " with value of: " .. optionValue);
	local inputCollection = parentElement:GetElementsByTagName(inputType);
	LogDebug(inputType .. " elements: " .. inputCollection.Count);
	local inputCollectionEnumerator = inputCollection:GetEnumerator();
	while inputCollectionEnumerator:MoveNext() do
		local element = inputCollectionEnumerator.Current;
		LogDebug("Got " .. element:GetAttribute("value") .. " option.");
		if element:GetAttribute("value") == optionValue then
			LogDebug("Found " .. optionValue .. " option.");
			element:SetAttribute(selectedAttribute, selectedAttribute); --checked or selected
			element:InvokeMember("click");
			break;
		end
	end
end

function PasteMemoText(inputElement, text)
	LogDebug("Attempting to paste text");
	inputElement.InnerText = text;
end

function NavigateToReceiptPage()
	local browser = GetBrowser();

	local aCollection = browser.Document:GetElementsByTagName("a");
	LogDebug("Count of Anchor tags is: " .. aCollection.Count);
	local receiptButton = nil;

	local aCollectionEnumerator = aCollection:GetEnumerator();
	while aCollectionEnumerator:MoveNext() do
		local element = aCollectionEnumerator.Current;
		LogDebug("Found anchor class: " .. element:GetAttribute("className"));
		if element:GetAttribute("className") == "usa-button-primary" then
			receiptButton = element;
			break;
		end
	end

	local link = receiptButton:GetAttribute("href");

	Navigate(link);
end


function CombineShipToAddress()
	_receiptTable["ShipTo"] = "";

	if _receiptTable["ShipToInstitution"] ~=nil then
		_receiptTable["ShipTo"] = _receiptTable["ShipTo"].._receiptTable["ShipToInstitution"].."/";
	end

	if _receiptTable["ShipToDepartment"] ~=nil then
		_receiptTable["ShipTo"] = _receiptTable["ShipTo"].._receiptTable["ShipToDepartment"].."/";
	end

	if _receiptTable["ShipToStreet1"]~= nil then
		_receiptTable["ShipTo"] = _receiptTable["ShipTo"].._receiptTable["ShipToStreet1"].."/";
	end

	if _receiptTable["ShipToStreet2"] ~= nil then
		_receiptTable["ShipTo"] = _receiptTable["ShipTo"].._receiptTable["ShipToStreet2"].."/";
	end

	if _receiptTable["ShipToLocation"] ~= nil then
		_receiptTable["ShipTo"] = _receiptTable["ShipTo"].._receiptTable["ShipToLocation"];
	end
end

function CombineBillToAddress()
	_receiptTable["BillTo"] = "";

	if _receiptTable["BillToInstitution"] ~=nil then
		_receiptTable["BillTo"] = _receiptTable["BillTo"].._receiptTable["BillToInstitution"].."/";
	end

	if _receiptTable["BillToDepartment"] ~=nil then
		_receiptTable["BillTo"] = _receiptTable["BillTo"].._receiptTable["BillToDepartment"].."/";
	end

	if _receiptTable["BillToStreet1"]~= nil then
		_receiptTable["BillTo"] = _receiptTable["BillTo"].._receiptTable["BillToStreet1"].."/";
	end

	if _receiptTable["BillToStreet2"] ~= nil then
		_receiptTable["BillTo"] = _receiptTable["BillTo"].._receiptTable["BillToStreet2"].."/";
	end

	if _receiptTable["BillToLocation"] ~= nil then
		_receiptTable["BillTo"] = _receiptTable["BillTo"].._receiptTable["BillToLocation"];
	end
end

function ParseRows(tdCollection)
	local tdCollectionEnumerator = tdCollection:GetEnumerator();
	while tdCollectionEnumerator:MoveNext() do
		local element = tdCollectionEnumerator.Current;
		local className = element:GetAttribute("className");

		--Only process for value tables with class names
		if className ~= "" then
			LogDebug("Found cell class: " .. className);

			if string.match(className, "req%-barcode") then
				_receiptTable["ID"] = string.match(element.InnerText, "(%d+)");
			elseif string.match(className, "req%-institution") then
				_receiptTable["ShipToInstitution"] = element.InnerText;
			elseif string.match(className, "req%-department") then
				_receiptTable["ShipToDepartment"] = element.InnerText;
			elseif string.match(className,"req%-street1") then
				_receiptTable["ShipToStreet1"] = element.InnerText;
			elseif string.match(className, "req%-street2") then
				_receiptTable["ShipToStreet2"] = element.InnerText;
			elseif string.match(className, "req%-location") then
				_receiptTable["ShipToLocation"] = element.InnerText;
			elseif string.match(className, "req%-journaltitleabbrev") then
				_receiptTable["TitleAbbreviation"] = element.InnerText;
			elseif string.match(className, "req%-journaltitle") then
				_receiptTable["Title"] = element.InnerText;
			elseif string.match(className, "req%-articletitle") then
				_receiptTable["Article"] = element.InnerText;
			elseif string.match(className, "req%-citation") and element.InnerText ~= nil then
				_receiptTable["Year"] = string.match(element.InnerText, "(%d+)");
				_receiptTable["Month"] = string.match(element.InnerText, "%d+%s*(%a+)%s*.*;");
				_receiptTable["Volume"] = string.match(element.InnerText, ";(%d+)%(");
				_receiptTable["Issue"] = string.match(element.InnerText, ";.*%((%d+)");
				local pageStart, pageEnd = string.match(element.InnerText, ":(%d+)%-(%d+)");
				if pageStart ~= nil and pageEnd ~= nil then
					_receiptTable["Pages"] = pageStart .."-"..pageEnd;
				end
				local doi = string.match(element.InnerText, "doi: (.*)");
				if doi ~= nil then
					_receiptTable["DOI"] = doi;
				end
			elseif string.match(className, "req%-authors") then
				_receiptTable["Authors"] = element.InnerText;
			elseif string.match(className, "bold req%-billlibid") then
				_receiptTable["Borrower"] = string.match(element.InnerText, "Bill to: (%u+)");
			elseif string.match(className, "bold req%-maxcost") then
				_receiptTable["MaxCost"] = element.InnerText;
			elseif string.match(className, "req%-issns") and element.InnerText ~= nil then
				local issnStart, issnEnd = string.match(element.InnerText, "(%d+)%-(%d+[xX]?).*%(Print%)");
				if issnStart ~=nil and issnEnd ~= nil then
					_receiptTable["ISSN"] = issnStart .."-"..issnEnd;
				end
				local eIssnStart, eIssnEnd = string.match(element.InnerText, "(%d+)%-(%d+[xX]?)%s*%(Electronic%)");
				if eIssnStart ~=nil and eIssnEnd ~= nil then
					_receiptTable["ESSN"] = eIssnStart .."-"..eIssnEnd;
				end
			elseif string.match(className, "req%-special") then
				if string.match(element.InnerText, "Rush") then
					_receiptTable["Rush"] = "Rush";
				elseif string.match(element.InnerText, "Urgent Patient Care") then
					_receiptTable["Rush"] = "Urgent Patient Care";
				else
					_receiptTable["Rush"] = "";
				end

				if string.match(element.InnerText, "Copy") then
					_receiptTable["ServiceType"] = "Copy";
				else
					_receiptTable["ServiceType"] = "Loan";
				end
			elseif string.match(className, "req%-publisher") then
				_receiptTable["Publisher"] = element.InnerText;
			elseif string.match(className, "req%-odyssey") then
				_receiptTable["OdysseyAddress"] = element.InnerText;
			elseif string.match(className, "req%-pmid") then
				_receiptTable["PMID"] = element.InnerText;
			elseif string.match(className, "req%-comments") then
				_receiptTable["Comments"] = element.InnerText;
			elseif string.match(className, "req%-copyright") then
				if string.match(element.InnerText,"Copyright Compliance Guidelines") then
					_receiptTable["CopyrightCompliance"] = "CCG";
				else
					_receiptTable["CopyrightCompliance"] = "CCL";
				end
			elseif string.match(className, "req%-needby") then
				_receiptTable["NeedBy"] = element.InnerText
			elseif string.match(className, "req%-billinstitution") then
				_receiptTable["BillToInstitution"] = element.InnerText;
			elseif string.match(className, "req%-billdepartment") then
				_receiptTable["BillToDepartment"] = element.InnerText;
			elseif string.match(className, "req%-billstreet1") then
				_receiptTable["BillToStreet1"] = element.InnerText;
			elseif string.match(className, "req%-billstreet2") then
				_receiptTable["BillToStreet2"] = element.InnerText;
			elseif string.match(className, "req%-billlocation") then
				_receiptTable["BillToLocation"] = element.InnerText;
			elseif (className ~= nil) and (element.InnerText ~= nil) and (string.match(className, "req%-patron")) then
				_receiptTable["Patron"] = element.InnerText;
				if (string.match(element.InnerText, "TN: (%d+)") ~= nil) then
					_receiptTable["BorrowerTN"] = string.match(element.InnerText, "TN: (%d+)");
				end
			end
		end
	end

	if ((_receiptTable["TitleAbbreviation"]) and ((_receiptTable["Title"] == nil) or (_receiptTable["Title"] == ""))) then
		_receiptTable["Title"] = _receiptTable["TitleAbbreviation"];
		LogDebug("Using the Title Abbreviation because the full title is not available.");
	end

	if ((_receiptTable["ESSN"]) and ((_receiptTable["ISSN"] == nil) or (_receiptTable["ISSN"] == ""))) then
		_receiptTable["ISSN"] = _receiptTable["ESSN"];
		LogDebug("Using the Electronic ISSN because the print ISSN is not available.");
	end

	CombineShipToAddress();
	CombineBillToAddress();
end

function ParseReceiptPage(importElement)
	LogDebug("Parsing the Receipts Page");

	local tdCollection = importElement:GetElementsByTagName("td");
	LogDebug("Count of td tags is: " .. tdCollection.Count);

	ParseRows(tdCollection);

	LogDebug("Printing the receipt table pairs");
	for key, value in pairs(_receiptTable) do
		if value then
			LogDebug(key..": ".. value);
		else
			LogDebug("Unable to parse a value for: ".. key);
		end
	end
end

function ParseSuccessPage(anchorCollection)
	local anchorCollectionEnumerator = anchorCollection:GetEnumerator();
	while anchorCollectionEnumerator:MoveNext() do
		local element = anchorCollectionEnumerator.Current;
		LogDebug("Found innerText of H3: " .. element.InnerHtml);
		if string.match(element.InnerHtml, "ID: (%d+)") then
			_receiptTable["ID"] = string.match(element.InnerHtml, "ID: (%d+)");
		end
		if string.match(element.InnerHtml, "/docline/library/libid/(%u+)") then
			_receiptTable["Lender"] = string.match(element.InnerHtml, "/docline/library/libid/(%u+)");
		end
	end
end

function ImportReceiptPage()
	local browser = GetBrowser();

	ParseReceiptPage(browser.Document);

	--import data from Docline to ILLiad
	local requestText = "";
	local tableCollection = browser.Document:GetElementsByTagName("TABLE");
	local tableCollectionEnumerator = tableCollection:GetEnumerator();
	while tableCollectionEnumerator:MoveNext() do
		local element = tableCollectionEnumerator.Current;
		LogDebug("Found TABLE class: " .. element:GetAttribute("className"));
		if string.match(element:GetAttribute("className"), "request%-table") then
			--Process table
			requestText = element.innerText;
			break;
		end
	end

	DoTransactionImport(_receiptTable, requestText);
	_receiptTable = {};
end